@@ -30,6 +30,7 @@
 #include <parse-util.h>
 #include <dependent.h>
 #include <cell.h>
+#include <collect.h>
 #include <sheet.h>
 #include <value.h>
 #include <ranges.h>
@@ -1336,8 +1337,8 @@ static GnmFuncHelp const help_column[] = {
 	{ GNM_FUNC_HELP_ARG, F_("x:reference, defaults to the position of the current expression") },
 	{ GNM_FUNC_HELP_DESCRIPTION, F_("COLUMN function returns a Nx1 array containing the sequence "
 					"of integers "
-					"from the first column to the last column of @{reference}.") },
-	{ GNM_FUNC_HELP_NOTE, F_("If @{reference} is neither an array nor a reference nor a range, "
+					"from the first column to the last column of @{x}.") },
+	{ GNM_FUNC_HELP_NOTE, F_("If @{x} is neither an array nor a reference nor a range, "
 				 "returns #VALUE!") },
 	{ GNM_FUNC_HELP_EXAMPLES, "=COLUMN(A1:C4)" },
 	{ GNM_FUNC_HELP_EXAMPLES, "=COLUMN(A:C)" },
@@ -1488,8 +1489,8 @@ static GnmFuncHelp const help_row[] = {
 	{ GNM_FUNC_HELP_ARG, F_("x:reference, defaults to the position of the current expression") },
 	{ GNM_FUNC_HELP_DESCRIPTION, F_("ROW function returns a 1xN array containing the "
 					"sequence of integers "
-					"from the first row to the last row of @{reference}.") },
-	{ GNM_FUNC_HELP_NOTE, F_("If @{reference} is neither an array nor a reference nor a range, "
+					"from the first row to the last row of @{x}.") },
+	{ GNM_FUNC_HELP_NOTE, F_("If @{x} is neither an array nor a reference nor a range, "
 				 "returns #VALUE!") },
 	{ GNM_FUNC_HELP_EXAMPLES, "=ROW(A1:D3)" },
 	{ GNM_FUNC_HELP_EXAMPLES, "=ROW(1:3)" },
@@ -1572,7 +1573,7 @@ gnumeric_sheets (GnmFuncEvalInfo *ei, GnmValue const * const *args)
 
 			a = g_slist_index (workbook_sheets (wb), r->a.sheet);
 			b = g_slist_index (workbook_sheets (wb), r->b.sheet);
-			
+
 			ans_min = (a < b) ? a : b;
 			ans_max = (a < b) ? b : a;
 
@@ -1590,8 +1591,9 @@ gnumeric_sheets (GnmFuncEvalInfo *ei, GnmValue const * const *args)
 static GnmFuncHelp const help_sheet[] = {
 	{ GNM_FUNC_HELP_NAME, F_("SHEET:sheet number of @{reference}")},
         { GNM_FUNC_HELP_ARG, F_("reference:reference or literal sheet name, defaults to the current sheet")},
-	{ GNM_FUNC_HELP_NOTE, F_("If @{reference} is neither a reference nor a literal sheet name, "
-				 "SHEETS returns #VALUE!")},
+	{ GNM_FUNC_HELP_NOTE, F_("If @{reference} is neither a reference "
+				 "nor a literal sheet name, "
+				 "SHEET returns #VALUE!")},
         { GNM_FUNC_HELP_EXAMPLES, "=SHEET()" },
         { GNM_FUNC_HELP_EXAMPLES, "=SHEET(\"Sheet1\")" },
         { GNM_FUNC_HELP_SEEALSO, "SHEETS,ROW,COLUMNNUMBER"},
@@ -1630,7 +1632,7 @@ gnumeric_sheet (GnmFuncEvalInfo *ei, GnmValue const * const *args)
 					(1 + g_slist_index (workbook_sheets (wb), 
 							    sheet));
 		} else
-			return value_new_error_VALUE (ei->pos);	
+			return value_new_error_VALUE (ei->pos);
 	} else
 		return value_new_int (1 + g_slist_index (workbook_sheets (wb), 
 							 ei->pos->sheet));
@@ -1695,6 +1697,113 @@ gnumeric_transpose (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 	return res;
 }
 
+/***************************************************************************/
+static GnmFuncHelp const help_array[] = {
+        { GNM_FUNC_HELP_NAME, F_("ARRAY:vertical array of the arguments")},
+        { GNM_FUNC_HELP_ARG, F_("v:value")},
+        { GNM_FUNC_HELP_SEEALSO, "TRANSPOSE"},
+        { GNM_FUNC_HELP_END}
+};
+
+
+static GnmValue *
+callback_function_array (GnmEvalPos const *ep, GnmValue const *value, void *closure)
+{
+	GSList **list = closure;
+
+	*list = g_slist_prepend 
+		(*list, value ? value_dup (value) : value_new_empty ());
+	return NULL;
+}
+
+static GnmValue *
+gnumeric_array (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
+{
+	GSList *list = NULL, *l;
+	int len, i;
+	GnmValue *val = function_iterate_argument_values 
+		(ei->pos, callback_function_array, &list,
+		 argc, argv, FALSE, CELL_ITER_ALL);
+
+	if (val != NULL) {
+		go_slist_free_custom (list, (GFreeFunc) value_release);
+		return val;
+	}
+	list = g_slist_reverse (list);
+	len = g_slist_length (list);
+
+	if (len == 0) {
+		go_slist_free_custom (list, (GFreeFunc) value_release);
+		return value_new_error_VALUE (ei->pos);
+	}
+		
+	if (len == 1) {
+		val = list->data;
+		g_slist_free (list);
+		return val;
+	}
+
+	val = value_new_array_empty (1, len);
+	
+	for (l = list, i = 0; l != NULL; l = l->next, i++)
+		val->v_array.vals[0][i] = l->data;
+
+	g_slist_free (list);
+	return val;
+}
+
+
+/***************************************************************************/
+
+static GnmFuncHelp const help_sort[] = {
+	{ GNM_FUNC_HELP_NAME, F_("SORT:sorted list of numbers as vertical array")},
+	{ GNM_FUNC_HELP_ARG, F_("ref:list of numbers")},
+	{ GNM_FUNC_HELP_ARG, F_("order:0 (descending order) or 1 (ascending order); defaults to 0")},
+	{ GNM_FUNC_HELP_NOTE, F_("Strings, booleans, and empty cells are ignored.")},
+	{ GNM_FUNC_HELP_EXAMPLES, F_("SORT({4,3,5}) evaluates to {5,4,3}")},
+	{ GNM_FUNC_HELP_SEEALSO, ("ARRAY")},
+	{ GNM_FUNC_HELP_END }
+};
+
+static GnmValue *
+gnumeric_sort (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
+{
+	gnm_float *xs;
+	int i, j, n;
+	GnmValue *result = NULL;
+
+	xs = collect_floats_value (argv[0], ei->pos,
+				   COLLECT_IGNORE_STRINGS |
+				   COLLECT_IGNORE_BOOLS |
+				   COLLECT_IGNORE_BLANKS |
+				   COLLECT_SORT,
+				   &n, &result);
+	if (result)
+		goto out;
+
+	switch (argv[1] ? value_get_as_int (argv[1]) : 0) {
+	case 0:
+		result = value_new_array_empty (1, n);
+		
+		for (i = 0, j = n - 1; i < n; i++, j--)
+			result->v_array.vals[0][i] = value_new_float (xs[j]);
+		break;
+	case 1:
+		result = value_new_array_empty (1, n);
+		
+		for (i = 0; i < n; i++)
+			result->v_array.vals[0][i] = value_new_float (xs[i]);
+		break;
+	default:
+		result = value_new_error_VALUE (ei->pos);
+		break;
+	}
+
+ out:
+	g_free (xs);
+
+	return result;
+}
 /***************************************************************************/
 
 GnmFuncDescriptor const lookup_functions[] = {
@@ -1749,12 +1858,18 @@ GnmFuncDescriptor const lookup_functions[] = {
 	{ "sheet",      "|?",
 	  help_sheet,     gnumeric_sheet, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
+	{ "sort",         "r|f",
+	  help_sort, gnumeric_sort, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_RETURNS_NON_SCALAR, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
 	{ "transpose", "A",
 	  help_transpose, gnumeric_transpose, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_RETURNS_NON_SCALAR, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "vlookup",   "EAf|bb",
 	  help_vlookup, gnumeric_vlookup, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "array", NULL,
+	  help_array, NULL, gnumeric_array, NULL, NULL, NULL,
+	  GNM_FUNC_RETURNS_NON_SCALAR, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
 
         {NULL}
 };
