@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * html_read.c
  *
@@ -44,9 +45,7 @@
 #include <hlink.h>
 #include <cell.h>
 #include <ranges.h>
-#include <goffice/app/io-context.h>
-#include <goffice/app/error-info.h>
-#include <goffice/utils/go-glib-extras.h>
+#include <goffice/goffice.h>
 
 #include <gsf/gsf-input.h>
 #include <libxml/HTMLparser.h>
@@ -60,8 +59,14 @@
 typedef struct {
 	Sheet *sheet;
 	int   row;
+	WorkbookView *wb_view;
 } GnmHtmlTableCtxt;
 
+static void html_read_table (htmlNodePtr cur, htmlDocPtr doc, 
+			     WorkbookView *wb_view,
+			     GnmHtmlTableCtxt *tc);
+
+
 static Sheet *
 html_get_sheet (char const *name, Workbook *wb)
 {
@@ -102,7 +107,7 @@ html_append_text (GString *buf, const xmlChar *text)
 static void
 html_read_content (htmlNodePtr cur, GString *buf, GnmStyle *mstyle,
 		   xmlBufferPtr a_buf, GSList **hrefs, gboolean first,
-		   htmlDocPtr doc)
+		   htmlDocPtr doc, GnmHtmlTableCtxt *tc)
 {
 	htmlNodePtr ptr;
 
@@ -141,8 +146,20 @@ html_read_content (htmlNodePtr cur, GString *buf, GnmStyle *mstyle,
 					props = props->next;
 				}
 			}
-			html_read_content (
-				ptr, buf, mstyle, a_buf, hrefs, first, doc);
+			if (xmlStrEqual (ptr->name, CC2XML ("table"))) {
+				Sheet *last_sheet = tc->sheet;
+				int   last_row = tc->row;
+				tc->sheet = NULL;
+				tc->row   = -1;
+				html_read_table (ptr, doc, tc->wb_view, tc);
+				g_string_append_printf (buf, _("[see sheet %s]"), tc->sheet->name_quoted);
+				xmlBufferAdd (a_buf, CC2XML (_("The original html file is\n"
+							       "using nested tables.")), -1);
+				tc->sheet = last_sheet;
+				tc->row = last_row;
+			} else
+				html_read_content 
+					(ptr, buf, mstyle, a_buf, hrefs, first, doc, tc);
 		}
 		first = FALSE;
 	}
@@ -198,7 +215,7 @@ html_read_row (htmlNodePtr cur, htmlDocPtr doc, GnmHtmlTableCtxt *tc)
 				gnm_style_set_font_bold (mstyle, TRUE);
 
 			html_read_content (ptr, buf, mstyle, a_buf,
-					   &hrefs, TRUE, doc);
+					   &hrefs, TRUE, doc, tc);
 
 
 			if (g_slist_length (hrefs) >= 1 &&
@@ -465,7 +482,7 @@ html_search_for_tables (htmlNodePtr cur, htmlDocPtr doc,
 }
 
 void
-html_file_open (GOFileOpener const *fo, IOContext *io_context,
+html_file_open (GOFileOpener const *fo, GOIOContext *io_context,
 		WorkbookView *wb_view, GsfInput *input)
 {
 	guint8 const *buf;
@@ -511,7 +528,7 @@ html_file_open (GOFileOpener const *fo, IOContext *io_context,
 				bomlen = 0;
 				/* Try to detect unmarked UTF16LE
 				   (Firefox Windows clipboard, drag data all platforms) */
-				if ((buf[0] >= 0x20 || g_ascii_isspace(buf[0])) && 
+				if ((buf[0] >= 0x20 || g_ascii_isspace(buf[0])) &&
 				    buf[1] == 0 &&
 				    (buf[2] >= 0x20 || g_ascii_isspace(buf[2])) &&
 				    buf[3] == 0)
@@ -543,17 +560,18 @@ html_file_open (GOFileOpener const *fo, IOContext *io_context,
 		xmlNodePtr ptr;
 		tc.sheet = NULL;
 		tc.row   = -1;
+		tc.wb_view = wb_view;
 		for (ptr = doc->children; ptr != NULL ; ptr = ptr->next)
 			html_search_for_tables (ptr, doc, wb_view, &tc);
 		xmlFreeDoc (doc);
 	} else
-		gnumeric_io_error_info_set (io_context,
-			error_info_new_str (_("Unable to parse the html.")));
+		go_io_error_info_set (io_context,
+			go_error_info_new_str (_("Unable to parse the html.")));
 }
 
 /* Quick and dirty html probe. */
 gboolean
-html_file_probe (GOFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
+html_file_probe (GOFileOpener const *fo, GsfInput *input, GOFileProbeLevel pl)
 {
 	gsf_off_t size = 200;
 	guint8 const* buf = gsf_input_read (input, size, NULL);
