@@ -111,7 +111,8 @@ cell_calc_layout (GnmCell const *cell, GnmRenderedValue *rv, int y_direction,
 		if (sin_a < 0) {
 			hoffset += (width - indent) - rv->layout_natural_width;
 		}
-	} else if (!rv->rotation && rv->wrap_text) {
+	} else if (!rv->rotation && rv->wrap_text 
+		   && (rv->effective_halign != HALIGN_FILL)) {
 		int wanted_width = MAX (0, width - indent);
 		if (wanted_width != pango_layout_get_width (layout)) {
 			pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
@@ -132,7 +133,8 @@ cell_calc_layout (GnmCell const *cell, GnmRenderedValue *rv, int y_direction,
 		case HALIGN_CENTER_ACROSS_SELECTION:
 			hoffset += ((width - indent) - rv->layout_natural_width) / 2;
 			break;
-		case HALIGN_FILL:
+		case HALIGN_FILL: {
+			PangoDirection dir = PANGO_DIRECTION_LTR;
 			if (!rv->hfilled &&
 			    rv->layout_natural_width > 0 &&
 			    width - indent >= 2 * rv->layout_natural_width) {
@@ -145,6 +147,9 @@ cell_calc_layout (GnmCell const *cell, GnmRenderedValue *rv, int y_direction,
 				size_t len1 = strlen (copy1);
 				GString *multi = g_string_sized_new ((len1 + 6) * copies);
 				int i;
+				PangoAttrList *attr = pango_layout_get_attributes (layout);
+
+				dir = pango_find_base_dir (copy1, -1);
 				for (i = 0; i < copies; i++) {
 					if (i)
 						g_string_append_unichar (multi, UNICODE_ZERO_WIDTH_SPACE_C);
@@ -152,9 +157,27 @@ cell_calc_layout (GnmCell const *cell, GnmRenderedValue *rv, int y_direction,
 				}
 				pango_layout_set_text (layout, multi->str, multi->len);
 				g_string_free (multi, TRUE);
+
+				if (attr != NULL && !go_pango_attr_list_is_empty (attr)) {
+					PangoAttrList *attr_c = pango_attr_list_copy (attr);
+					size_t len = len1 + UNICODE_ZERO_WIDTH_SPACE_C_UTF8_LENGTH;
+					for (i = 1; i < copies;
+					     i++, len += len1 + UNICODE_ZERO_WIDTH_SPACE_C_UTF8_LENGTH)
+						pango_attr_list_splice (attr, attr_c, len, len1);
+					pango_attr_list_unref (attr_c);
+				}
+			} else
+				dir = pango_find_base_dir (pango_layout_get_text (layout), -1);
+			/* right align if text is RTL */
+			if (dir == PANGO_DIRECTION_RTL) {
+				PangoRectangle r;
+				pango_layout_get_extents (layout, NULL, &r);
+				hoffset += (width - indent) - r.width;
 			}
+			
 			rv->hfilled = TRUE;
 			break;
+		}
 
 #ifndef DEBUG_SWITCH_ENUM
 		default:
