@@ -102,7 +102,7 @@ ColRowIndexList *colrow_get_index_list	     (int first, int last,
 					      ColRowIndexList *list);
 ColRowIndexList *colrow_index_list_copy	     (ColRowIndexList *list);
 
-#define colrow_state_list_destroy(l) (go_slist_free_custom ((l), g_free), NULL)
+ColRowStateList *colrow_state_list_destroy   (ColRowStateList *list);
 
 ColRowStateList	*colrow_get_states	     (Sheet *sheet, gboolean is_cols,
 					      int first, int last);
@@ -111,7 +111,8 @@ void		 colrow_set_states	     (Sheet *sheet, gboolean is_cols,
 
 ColRowStateGroup  *colrow_state_group_destroy	(ColRowStateGroup *set);
 ColRowStateGroup  *colrow_set_sizes		(Sheet *sheet, gboolean is_cols,
-						 ColRowIndexList *src, int new_size);
+						 ColRowIndexList *src, int new_size, 
+						 int from, int to);
 ColRowStateGroup  *colrow_get_sizes		(Sheet *sheet, gboolean is_cols,
 						 ColRowIndexList *src, int new_size);
 void		   colrow_restore_state_group	(Sheet *sheet, gboolean is_cols,
@@ -154,6 +155,8 @@ void             colrow_autofit                 (Sheet *sheet,
 						 gboolean min_default,
 						 ColRowIndexList **indices,
 						 ColRowStateList **sizes);
+void             colrow_autofit_col             (Sheet *sheet, GnmRange *r);
+void             colrow_autofit_row             (Sheet *sheet, GnmRange *r);
 
 G_END_DECLS
 
