@@ -1,6 +1,6 @@
 /* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 #ifndef _GNM_COMMANDS_H_
-# define _GNM_COMMANDS_H_
+#define _GNM_COMMANDS_H_
 
 #include "gnumeric.h"
 #include "tools/tools.h"
@@ -13,12 +13,34 @@ void command_repeat (WorkbookControl *wbc);
 void command_setup_combos	(WorkbookControl *wbc);
 void command_list_release	(GSList *cmds);
 
+/* utility functions */
+
+GString *gnm_cmd_trunc_descriptor (GString *src, gboolean *truncated);
+gboolean cmd_cell_range_is_locked_effective (Sheet *sheet, GnmRange *range,
+					     WorkbookControl *wbc, 
+					     char const *cmd_name);
+gboolean cmd_selection_is_locked_effective (Sheet *sheet, GSList *selection,
+					    WorkbookControl *wbc, 
+					    char const *cmd_name);
+
+/* Commands: note that any extensions should ideally use cmd_generic* */
+
+gboolean cmd_generic (WorkbookControl *wbc, const char *text,
+		      GOUndo *undo, GOUndo *redo);
+gboolean cmd_generic_with_size (WorkbookControl *wbc, const char *text,
+				int size,
+				GOUndo *undo, GOUndo *redo);
+
 gboolean cmd_set_text		(WorkbookControl *wbc, Sheet *sheet,
 				 GnmCellPos const *pos, char const *new_text,
 				 PangoAttrList *markup);
 
 gboolean cmd_area_set_text	(WorkbookControl *wbc, SheetView *sv,
-				 char const *text, gboolean as_array);
+				 char const *text, PangoAttrList *markup);
+
+gboolean cmd_area_set_array_expr (WorkbookControl *wbc, SheetView *sv,
+				  GnmExprTop const  *new_texpr);
+
 gboolean cmd_create_data_table	(WorkbookControl *wbc,
 				 Sheet *sheet, GnmRange const *r,
 				 char const *col_input, char const *row_input);
@@ -35,6 +57,9 @@ gboolean cmd_delete_rows	(WorkbookControl *wbc, Sheet *sheet,
 gboolean cmd_resize_colrow	(WorkbookControl *wbc, Sheet *sheet,
 				 gboolean is_col, ColRowIndexList *selection,
 				 int new_size);
+gboolean cmd_autofit_selection  (WorkbookControl *wbc, SheetView *sv, Sheet *sheet, 
+				 gboolean fit_width, 
+				 ColRowIndexList *selectionlist);
 
 gboolean cmd_paste_cut		(WorkbookControl *wbc,
 				 GnmExprRelocateInfo const *info,
@@ -61,6 +86,8 @@ gboolean cmd_copyrel		(WorkbookControl *wbc,
 gboolean cmd_selection_format		(WorkbookControl *wbc,
 					 GnmStyle *style, GnmBorder **borders,
 					 char const *opt_translated_name);
+gboolean cmd_selection_format_toggle_font_style (WorkbookControl *wbc,
+						 GnmStyle *style, GnmStyleElement t);
 gboolean cmd_selection_clear		(WorkbookControl *wbc, int clear_flags);
 gboolean cmd_selection_colrow_hide	(WorkbookControl *wbc,
 					 gboolean is_cols, gboolean visible);
@@ -111,6 +138,8 @@ gboolean cmd_define_name	(WorkbookControl *wbc, char const *name,
 				 GnmExprTop const *texpr,
 				 char const *descriptor);
 gboolean cmd_remove_name        (WorkbookControl *wbc, GnmNamedExpr *nexpr);
+gboolean cmd_rescope_name       (WorkbookControl *wbc, GnmNamedExpr *nexpr, 
+				 Sheet *scope);
 
 gboolean cmd_scenario_add (WorkbookControl *wbc, GnmScenario *s, Sheet *sheet);
 gboolean cmd_scenario_mngr (WorkbookControl *wbc, GnmScenario *sc,
@@ -126,9 +155,6 @@ gboolean cmd_text_to_columns (WorkbookControl *wbc,
 			      GnmRange const *target, Sheet *target_sheet,
 			      GnmCellRegion *content);
 
-gboolean cmd_solver (WorkbookControl *wbc, const char *text,
-		     GOUndo *undo, GOUndo *redo);
-
 gboolean cmd_goal_seek (WorkbookControl *wbc,
 			GnmCell *cell, GnmValue *ov, GnmValue *nv);
 
@@ -137,6 +163,9 @@ gboolean cmd_tabulate (WorkbookControl *wbc, gpointer data);
 gboolean cmd_toggle_rtl (WorkbookControl *wbc, Sheet *sheet);
 
 gboolean cmd_autofilter_add_remove (WorkbookControl *wbc);
+gboolean cmd_autofilter_set_condition (WorkbookControl *wbc, 
+				       GnmFilter *filter, unsigned i, 
+				       GnmFilterCondition *cond);
 
 /**************************  Sheet Objects **************************************/
 
@@ -148,8 +177,6 @@ gboolean cmd_objects_move	(WorkbookControl *wbc,
 gboolean cmd_object_format	(WorkbookControl *wbc, SheetObject *so,
 				 gpointer orig_style, char *orig_text,
 				 PangoAttrList *orig_attr);
-gboolean cmd_so_rename	        (WorkbookControl *wbc, SheetObject *so,
-				 char const *new_name);
 
 gboolean cmd_so_graph_config (WorkbookControl *wbc, SheetObject *sog,
                               GObject *n_graph, GObject *o_graph);
