@@ -31,6 +31,7 @@
 #include <workbook.h>
 #include <wbc-gtk.h>
 #include <sheet.h>
+#include <cell.h>
 #include <value.h>
 #include <sheet-filter.h>
 #include <number-match.h>
@@ -50,6 +51,8 @@ typedef struct {
 } AutoFilterState;
 
 #define DIALOG_KEY "autofilter"
+#define DIALOG_KEY_EXPRESSION "autofilter-expression"
+#define UNICODE_ELLIPSIS "\xe2\x80\xa6"
 
 static void
 cb_autofilter_destroy (AutoFilterState *state)
@@ -231,10 +234,15 @@ dialog_auto_filter (WBCGtk *wbcg,
 	AutoFilterState *state;
 	GtkWidget *w;
 	GladeXML *gui;
+	int col;
+	gchar *label;
+	GnmCell *cell;
+	int len = is_expr ? 15 : 30;
 
 	g_return_if_fail (wbcg != NULL);
 
-	if (gnumeric_dialog_raise_if_exists (wbcg, DIALOG_KEY))
+	if (gnumeric_dialog_raise_if_exists 
+	    (wbcg, is_expr ? DIALOG_KEY_EXPRESSION : DIALOG_KEY))
 		return;
 	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 			(is_expr ? "autofilter-expression.glade" : "autofilter-top10.glade"),
@@ -251,13 +259,37 @@ dialog_auto_filter (WBCGtk *wbcg,
 
 	g_return_if_fail (state->gui != NULL);
 
+	col = filter->r.start.col + field;
+
+	cell = sheet_cell_get (filter->sheet, col, filter->r.start.row);
+
+	if (cell == NULL || gnm_cell_is_blank (cell))
+		label = g_strdup_printf (_("Column %s"), col_name (col));
+	else {
+		char *content = gnm_cell_get_rendered_text (cell);
+		if (g_utf8_strlen (content, -1) > len) {
+			char *end = g_utf8_find_prev_char (content, content + len + 1 - strlen (UNICODE_ELLIPSIS));
+			strcpy (end, UNICODE_ELLIPSIS);
+		}
+		label = g_strdup_printf (_("Column %s (\"%s\")"), 
+					 col_name (col), content);
+		g_free (content);
+	}
+
 	if (is_expr) {
+		gtk_label_set_text 
+			(GTK_LABEL (glade_xml_get_widget (state->gui, "col-label1")), label);
+		gtk_label_set_text 
+			(GTK_LABEL (glade_xml_get_widget (state->gui, "col-label2")), label);
 	} else {
+		gtk_label_set_text 
+			(GTK_LABEL (glade_xml_get_widget (state->gui, "col-label")), label);
 		w = glade_xml_get_widget (state->gui, "item_vs_percentage_option_menu");
 		g_signal_connect (G_OBJECT (w),
 			"changed",
 			G_CALLBACK (cb_top10_type_changed), state);
 	}
+	g_free (label);
 
 	state->dialog = glade_xml_get_widget (state->gui, "dialog");
 	if (cond != NULL) {
@@ -319,6 +351,7 @@ dialog_auto_filter (WBCGtk *wbcg,
 	g_object_set_data_full (G_OBJECT (state->dialog),
 		"state", state, (GDestroyNotify)cb_autofilter_destroy);
 
-	gnumeric_keyed_dialog (wbcg, GTK_WINDOW (state->dialog), DIALOG_KEY);
+	gnumeric_keyed_dialog (wbcg, GTK_WINDOW (state->dialog), 
+			       is_expr ? DIALOG_KEY_EXPRESSION : DIALOG_KEY);
 	gtk_widget_show (state->dialog);
 }
