@@ -53,14 +53,12 @@ typedef struct {
 	RenderData_t  *renderdata;
 	int            context_col, context_dx;
 	int            ruler_x;
-	GdkGC         *ruler_gc;
 } FixedInfo_t;
 
 /* for the format_page */
 typedef struct {
 	GtkWidget         *format_data_container;
         GtkWidget	  *format_trim;
-	GOFormatSel	  *format_selector;
 	GtkWidget         *column_selection_label;
 
 	/* Page members that are created at run-time */
@@ -75,7 +73,6 @@ typedef struct {
         int                col_import_count;
         int                col_import_array_len;
         char              *col_header;
-        gulong             format_changed_handler_id;
 } FormatInfo_t;
 
 
