@@ -27,14 +27,12 @@
 #include "mstyle.h"
 #include "gutils.h"
 #include "sheet.h"
-#include "sheet-style.h"
-#include "style-border.h"
 #include "command-context.h"
 #include "ranges.h"
-#include "xml-io.h"
+#include "xml-sax.h"
 #include <goffice/goffice.h>
 #include <string.h>
-#include <libxml/parser.h>
+#include <gsf/gsf-input-stdio.h>
 
 #define CC2XML(s) ((xmlChar const *)(s))
 #define CXML2C(s) ((char const *)(s))
@@ -45,8 +43,6 @@ attr_eq (const xmlChar *a, const char *s)
 	return !strcmp (CXML2C (a), s);
 }
 
-#define ROW_COL_KEY(row,col) GINT_TO_POINTER ((guint)row * (guint)GNM_MAX_COLS + (guint)col)
-
 /******************************************************************************
  * FormatTemplateMember - Getters/setters and creation
  ******************************************************************************/
@@ -87,9 +83,7 @@ format_template_member_new (void)
 TemplateMember *
 format_template_member_clone (TemplateMember *member)
 {
-	TemplateMember *clone;
-
-	clone = format_template_member_new ();
+	TemplateMember *clone = format_template_member_new ();
 
 	clone->row = member->row;
 	clone->col = member->col;
@@ -187,44 +181,18 @@ format_template_member_get_rect (TemplateMember const *member, GnmRange const *r
 	return res;
 }
 
-/******************************************************************************
- * Getters and setters for FormatTemplateMember
- *
- * NOTE : GnmStyle are taken care of internally, there is no
- *        need to unref or ref mstyle's manually.
- */
+/****************************************************************************/
 
-static void
-format_template_member_set_direction (TemplateMember *member, FreqDirection direction)
+static gboolean
+format_template_member_valid (TemplateMember const *member)
 {
-	g_return_if_fail (direction == FREQ_DIRECTION_NONE || direction == FREQ_DIRECTION_HORIZONTAL ||
-			  direction == FREQ_DIRECTION_VERTICAL);
-
-	member->direction = direction;
-}
-
-static void
-format_template_member_set_repeat (TemplateMember *member, int repeat)
-{
-	g_return_if_fail (repeat >= -1);
-
-	member->repeat = repeat;
-}
-
-static void
-format_template_member_set_skip (TemplateMember *member, int skip)
-{
-	g_return_if_fail (skip >= 0);
-
-	member->skip = skip;
-}
-
-static void
-format_template_member_set_edge (TemplateMember *member, int edge)
-{
-	g_return_if_fail (edge >= 0);
-
-	member->edge = edge;
+	return (member &&
+		member->mstyle &&
+		member->direction >= FREQ_DIRECTION_NONE &&
+		member->direction <= FREQ_DIRECTION_VERTICAL &&
+		member->repeat >= -1 &&
+		member->skip >= 0 &&
+		member->edge >= 0);
 }
 
 /******************************************************************************
@@ -245,7 +213,7 @@ format_template_new (void)
 
 	ft = g_new0 (GnmFormatTemplate, 1);
 
-	ft->filename    = g_strdup ("");
+	ft->filename    = NULL;
 	ft->author      = g_strdup (go_get_real_name ());
 	ft->name        = g_strdup (N_("Name"));
 	ft->description = g_strdup ("");
@@ -281,19 +249,13 @@ format_template_new (void)
 void
 format_template_free (GnmFormatTemplate *ft)
 {
-	GSList *ptr;
-
 	g_return_if_fail (ft != NULL);
 
 	g_free (ft->filename);
 	g_free (ft->author);
 	g_free (ft->name);
 	g_free (ft->description);
-
-	for (ptr = ft->members; ptr != NULL ; ptr = ptr->next)
-		format_template_member_free (ptr->data);
-	g_slist_free (ft->members);
-
+	go_slist_free_custom (ft->members, (GFreeFunc)format_template_member_free);
 	g_hash_table_destroy (ft->table);
 
 	g_free (ft);
@@ -311,7 +273,6 @@ GnmFormatTemplate *
 format_template_clone (GnmFormatTemplate const *ft)
 {
 	GnmFormatTemplate *clone;
-	GSList *ptr = NULL;
 
 	g_return_val_if_fail (ft != NULL, NULL);
 
@@ -323,21 +284,15 @@ format_template_clone (GnmFormatTemplate const *ft)
 
 	clone->category    = ft->category;
 
-	for (ptr = ft->members; ptr != NULL ; ptr = ptr->next)
-		format_template_attach_member (clone,
-			format_template_member_clone (ptr->data));
+	clone->members = go_slist_map (ft->members,
+				       (GOMapFunc)format_template_member_clone);
 
 	clone->number    = ft->number;
 	clone->border    = ft->border;
 	clone->font      = ft->font;
 	clone->patterns  = ft->patterns;
 	clone->alignment = ft->alignment;
-
-	clone->edges.left   = ft->edges.left;
-	clone->edges.right  = ft->edges.right;
-	clone->edges.top    = ft->edges.top;
-	clone->edges.bottom = ft->edges.bottom;
-
+	clone->edges     = ft->edges;
 	clone->dimension = ft->dimension;
 
 	clone->invalidate_hash = TRUE;
@@ -345,107 +300,174 @@ format_template_clone (GnmFormatTemplate const *ft)
 	return clone;
 }
 
+#define GNM 100
+#define GMR 200
+
+static GsfXMLInNS const template_ns[] = {
+	GSF_XML_IN_NS (GMR, "http://www.gnome.org/gnumeric/format-template/v1"),
+	GSF_XML_IN_NS (GNM, "http://www.gnumeric.org/v10.dtd"),
+	GSF_XML_IN_NS_END
+};
+
 static void
-xml_read_format_col_row_info (FormatColRowInfo *info, xmlNodePtr parent)
+sax_information (GsfXMLIn *xin, xmlChar const **attrs)
 {
-	xmlNode *child;
-	int found = 0;
+	GnmFormatTemplate *ft = (GnmFormatTemplate *)xin->user_state;
 
-	for (child = parent->xmlChildrenNode; child != NULL ; child = child->next) {
-		if (xmlIsBlankNode (child) || child->name == NULL)
-			continue;
-		if (attr_eq (child->name, "Placement")) {
-			g_return_if_fail (!(found & 1));
-			go_xml_node_get_int  (child, "offset", &info->offset);
-			go_xml_node_get_int  (child, "offset_gravity", &info->offset_gravity);
-			found |= 1;
-		} else if (attr_eq (child->name, "Dimensions")) {
-			g_return_if_fail (!(found & 2));
-			go_xml_node_get_int (child, "size", &info->size);
-			found |= 2;
-		}
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (attr_eq (attrs[0], "author"))
+			format_template_set_author (ft, CXML2C (attrs[1]));
+		else if (attr_eq (attrs[0], "name"))
+			format_template_set_name (ft, CXML2C (attrs[1]));
+		else if (attr_eq (attrs[0], "description"))
+			format_template_set_description (ft, CXML2C (attrs[1]));
 	}
-	g_return_if_fail (found == 3);
 }
 
-static gboolean
-xml_read_format_template_member (XmlParseContext *ctxt, GnmFormatTemplate *ft, xmlNodePtr tree)
+static void
+sax_members_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	xmlNodePtr child;
-	TemplateMember *member;
-	int tmp, found = 0;
+	GnmFormatTemplate *ft = (GnmFormatTemplate *)xin->user_state;
+	ft->members = g_slist_reverse (ft->members);
+}
 
-	g_return_val_if_fail (attr_eq (tree->name, "Member"), FALSE);
-	member = format_template_member_new ();
+static void
+sax_member (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GnmFormatTemplate *ft = (GnmFormatTemplate *)xin->user_state;
+	TemplateMember *member = format_template_member_new ();
 
-	for (child = tree->xmlChildrenNode; child != NULL ; child = child->next) {
-		if (xmlIsBlankNode (child) || child->name == NULL)
-			continue;
-		if (attr_eq (child->name, "Col"))
-			xml_read_format_col_row_info (&member->col, child);
-		else if (attr_eq (child->name, "Row"))
-			xml_read_format_col_row_info (&member->row, child);
-		else if (attr_eq (child->name, "Frequency")) {
-			if (found & 1) { g_warning ("Multiple Frequency specs"); }
-			if (go_xml_node_get_int (child, "direction", &tmp))
-				format_template_member_set_direction (member, tmp);
-			if (go_xml_node_get_int (child, "repeat", &tmp))
-				format_template_member_set_repeat (member, tmp);
-			if (go_xml_node_get_int (child, "skip", &tmp))
-				format_template_member_set_skip (member, tmp);
-			if (go_xml_node_get_int (child, "edge", &tmp))
-				format_template_member_set_edge (member, tmp);
-			found |= 1;
-		} else if (attr_eq (child->name, "Style")) {
-			if (found & 2) { g_warning ("Multiple Styles"); }
-			member->mstyle = xml_read_style (ctxt, child, FALSE);
-			found |= 2;
-		}
-	}
-	if (found != 3) {
-		g_warning ("Invalid Member, missing %s", (found & 1) ? "Style" : "Frequency");
+	/* Order reversed in sax_members_end.  */
+	ft->members = g_slist_prepend (ft->members, member);
+}
+
+static void
+sax_member_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
+{
+	GnmFormatTemplate *ft = (GnmFormatTemplate *)xin->user_state;
+	TemplateMember *member = ft->members->data;
+
+	if (!format_template_member_valid (member)) {
+		g_warning ("Invalid template member in %s\n", ft->filename);
+		ft->members = g_slist_remove (ft->members, member);
 		format_template_member_free (member);
-		return FALSE;
 	}
+}
 
-	format_template_attach_member (ft, member);
-	return TRUE;
+static void
+sax_placement (FormatColRowInfo *info, xmlChar const **attrs)
+{
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (gnm_xml_attr_int (attrs, "offset", &info->offset) ||
+		    gnm_xml_attr_int (attrs, "offset_gravity", &info->offset_gravity))
+			; /* Nothing */
+	}
+}
+
+static void
+sax_row_placement (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GnmFormatTemplate *ft = (GnmFormatTemplate *)xin->user_state;
+	TemplateMember *member = ft->members->data;
+	sax_placement (&member->row, attrs);
+}
+
+static void
+sax_col_placement (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GnmFormatTemplate *ft = (GnmFormatTemplate *)xin->user_state;
+	TemplateMember *member = ft->members->data;
+	sax_placement (&member->col, attrs);
+}
+
+static void
+sax_dimensions (FormatColRowInfo *info, xmlChar const **attrs)
+{
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (gnm_xml_attr_int (attrs, "size", &info->size))
+			; /* Nothing */
+	}
+}
+
+static void
+sax_row_dimensions (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GnmFormatTemplate *ft = (GnmFormatTemplate *)xin->user_state;
+	TemplateMember *member = ft->members->data;
+	sax_dimensions (&member->row, attrs);
+}
+
+static void
+sax_col_dimensions (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GnmFormatTemplate *ft = (GnmFormatTemplate *)xin->user_state;
+	TemplateMember *member = ft->members->data;
+	sax_dimensions (&member->col, attrs);
+}
+
+static void
+sax_frequency (GsfXMLIn *xin, xmlChar const **attrs)
+{
+	GnmFormatTemplate *ft = (GnmFormatTemplate *)xin->user_state;
+	TemplateMember *member = ft->members->data;
+
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		int i;
+
+		if (gnm_xml_attr_int (attrs, "direction", &i))
+			member->direction = i;
+		else if (gnm_xml_attr_int (attrs, "repeat", &member->repeat) ||
+			 gnm_xml_attr_int (attrs, "skip", &member->skip) ||
+			 gnm_xml_attr_int (attrs, "edge", &member->edge))
+			; /* Nothing */
+	}
+}
+
+static void
+sax_style_handler (GsfXMLIn *xin, GnmStyle *style, gpointer user)
+{
+	GnmFormatTemplate *ft = (GnmFormatTemplate *)xin->user_state;
+	TemplateMember *member = ft->members->data;
+	gnm_style_ref (style);
+	member->mstyle = style;
 }
 
 static gboolean
-xml_read_format_template_members (XmlParseContext *ctxt, GnmFormatTemplate *ft, xmlNodePtr tree)
+template_sax_unknown (GsfXMLIn *xin, xmlChar const *elem, xmlChar const **attrs)
 {
-	xmlNode *child;
+	g_return_val_if_fail (xin != NULL, FALSE);
+	g_return_val_if_fail (xin->doc != NULL, FALSE);
+	g_return_val_if_fail (xin->node != NULL, FALSE);
 
-	g_return_val_if_fail (attr_eq (tree->name, "FormatTemplate"), FALSE);
-
-	child = go_xml_get_child_by_name_by_lang (tree, "Information");
-	if (child) {
-		xmlChar *author = go_xml_node_get_cstr (child, "author");
-		xmlChar *name   = go_xml_node_get_cstr (child, "name");
-		xmlChar *descr  = go_xml_node_get_cstr (child, "description");
-
-		format_template_set_author (ft, _(CXML2C (author)));
-		format_template_set_name (ft,  _(CXML2C (name)));
-		format_template_set_description (ft,  _(CXML2C (descr)));
-
-		xmlFree (author);
-		xmlFree (name);
-		xmlFree (descr);
-	} else
-		return FALSE;
-
-	child = go_xml_get_child_by_name (tree, "Members");
-	if (child == NULL)
-		return FALSE;
-	for (child = child->xmlChildrenNode; child != NULL ; child = child->next)
-		if (!xmlIsBlankNode (child) &&
-		    !xml_read_format_template_member (ctxt, ft, child))
-			return FALSE;
-
-	return TRUE;
+	if (GMR == xin->node->ns_id &&
+	    0 == strcmp (xin->node->id, "MEMBERS_MEMBER")) {
+		char const *type_name = gsf_xml_in_check_ns (xin, CXML2C (elem), GNM);
+		if (type_name && strcmp (type_name, "Style") == 0) {
+			gnm_xml_prep_style_parser (xin, attrs,
+						   sax_style_handler,
+						   NULL);
+			return TRUE;
+		}
+	}
+	return FALSE;
 }
 
+static GsfXMLInNode template_dtd[] = {
+GSF_XML_IN_NODE_FULL (START, START, -1, NULL, GSF_XML_NO_CONTENT, FALSE, TRUE, NULL, NULL, 0),
+GSF_XML_IN_NODE (START, TEMPLATE, GMR, "FormatTemplate", GSF_XML_NO_CONTENT, NULL, NULL),
+  GSF_XML_IN_NODE (TEMPLATE, TEMPLATE_INFORMATION, GMR, "Information", GSF_XML_NO_CONTENT, sax_information, NULL),
+  GSF_XML_IN_NODE (TEMPLATE, TEMPLATE_MEMBERS, GMR, "Members", GSF_XML_NO_CONTENT, NULL, sax_members_end),
+    GSF_XML_IN_NODE (TEMPLATE_MEMBERS, MEMBERS_MEMBER, GMR, "Member", GSF_XML_NO_CONTENT, sax_member, sax_member_end),
+      GSF_XML_IN_NODE (MEMBERS_MEMBER, MEMBER_ROW, GMR, "Row", GSF_XML_NO_CONTENT, NULL, NULL),
+        GSF_XML_IN_NODE (MEMBER_ROW, ROW_PLACEMENT, GMR, "Placement", GSF_XML_NO_CONTENT, sax_row_placement, NULL),
+        GSF_XML_IN_NODE (MEMBER_ROW, ROW_DIMENSIONS, GMR, "Dimensions", GSF_XML_NO_CONTENT, sax_row_dimensions, NULL),
+      GSF_XML_IN_NODE (MEMBERS_MEMBER, MEMBER_COL, GMR, "Col", GSF_XML_NO_CONTENT, NULL, NULL),
+        GSF_XML_IN_NODE (MEMBER_COL, COL_PLACEMENT, GMR, "Placement", GSF_XML_NO_CONTENT, sax_col_placement, NULL),
+        GSF_XML_IN_NODE (MEMBER_COL, COL_DIMENSIONS, GMR, "Dimensions", GSF_XML_NO_CONTENT, sax_col_dimensions, NULL),
+      GSF_XML_IN_NODE (MEMBERS_MEMBER, MEMBER_FREQUENCY, GMR, "Frequency", GSF_XML_NO_CONTENT, sax_frequency, NULL),
+  GSF_XML_IN_NODE_END
+};
+
 /**
  * format_template_new_from_file:
  * @context:
@@ -459,161 +481,46 @@ xml_read_format_template_members (XmlParseContext *ctxt, GnmFormatTemplate *ft,
 GnmFormatTemplate *
 format_template_new_from_file (char const *filename, GOCmdContext *cc)
 {
-	GnmFormatTemplate	*ft = NULL;
-	xmlDoc		*doc;
+	GnmFormatTemplate *ft = NULL;
+	GsfXMLInDoc *doc = NULL;
+	GnmLocale *locale;
+	gboolean ok = FALSE;
+	GsfInput *input = NULL;
 
 	g_return_val_if_fail (filename != NULL, NULL);
 
-	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
-		return NULL;
-
-	doc = xmlParseFile (filename);
-	if (doc == NULL) {
-		go_cmd_context_error_import (cc,
-			_("Error while trying to load autoformat template"));
-		return NULL;
+	input = gsf_input_stdio_new (filename, NULL);
+	if (!input) {
+		go_cmd_context_error_import
+			(cc,
+			 _("Error while opening autoformat template"));
+		goto done;
 	}
-	if (doc->xmlRootNode != NULL) {
-		xmlNs *ns = xmlSearchNsByHref (doc, doc->xmlRootNode,
-			CC2XML ("http://www.gnome.org/gnumeric/format-template/v1"));
-		if (ns != NULL && attr_eq (doc->xmlRootNode->name, "FormatTemplate")) {
-			XmlParseContext *ctxt = xml_parse_ctx_new (doc, ns, NULL);
 
-			ft = format_template_new ();
-			if (xml_read_format_template_members (ctxt, ft, doc->xmlRootNode)) {
-				g_free (ft->filename);
-				ft->filename = g_strdup (filename);
-			} else {
-				format_template_free (ft);
-				ft = NULL;
-				go_cmd_context_error_import (cc,
-					_("Error while trying to build tree from autoformat template file"));
-			}
+	doc = gsf_xml_in_doc_new (template_dtd, template_ns);
+	if (doc == NULL)
+		goto done;
+	gsf_xml_in_doc_set_unknown_handler (doc, &template_sax_unknown);
 
-			xml_parse_ctx_destroy (ctxt);
-		} else
-			go_cmd_context_error_import (cc,
-				_("Is not an autoformat template file"));
-	} else
-		go_cmd_context_error_import (cc,
-			_("Invalid xml file. Tree is empty?"));
+	ft = format_template_new ();
+	ft->filename = g_strdup (filename);
+
+	locale = gnm_push_C_locale ();
+	ok = gsf_xml_in_doc_parse (doc, input, ft);
+	gnm_pop_C_locale (locale);
+
+ done:
+	if (input) g_object_unref (input);
+	if (doc) gsf_xml_in_doc_free (doc);
+
+	if (ft && !ok) {
+		format_template_free (ft);
+		ft = NULL;
+	}
 
-	xmlFreeDoc (doc);
 	return ft;
 }
 
-#if 0
-static xmlNode *
-format_colrow_info_write_xml (FormatColRowInfo const *info,
-			      xmlNode *parent, xmlChar const *type,
-			      G_GNUC_UNUSED XmlParseContext *ctxt)
-{
-	xmlNode *tmp, *container;
-
-	container = xmlNewChild (parent, parent->ns, type, NULL);
-	tmp = xmlNewChild (container, container->ns, CC2XML ("Placement"), NULL);
-	go_xml_node_set_int (tmp, "offset", info->offset);
-	go_xml_node_set_int (tmp, "offset_gravity", info->offset_gravity);
-
-	tmp = xmlNewChild (container, container->ns, CC2XML ("Dimensions"), NULL);
-	go_xml_node_set_int (tmp, "size", info->size);
-
-	return container;
-}
-
-/*
- * Create an XML subtree of doc equivalent to the given TemplateMember
- */
-static xmlNodePtr
-xml_write_format_template_member (XmlParseContext *ctxt, TemplateMember *member)
-{
-	xmlNode *tmp, *member_node;
-
-	member_node = xmlNewDocNode (ctxt->doc, ctxt->ns, CC2XML ("Member"), NULL);
-	if (member_node == NULL)
-		return NULL;
-
-	format_colrow_info_write_xml (&member->col, member_node, "Col", ctxt);
-	format_colrow_info_write_xml (&member->row, member_node, "Row", ctxt);
-
-	tmp = xmlNewChild (member_node, member_node->ns, CC2XML ("Frequency") , NULL);
-	go_xml_node_set_int (tmp, "direction", member->direction);
-	go_xml_node_set_int (tmp, "repeat", member->repeat);
-	go_xml_node_set_int (tmp, "skip", member->skip);
-	go_xml_node_set_int (tmp, "edge", member->edge);
-
-	xmlAddChild (member_node, xml_write_style (ctxt, member->mstyle));
-
-	return member_node;
-}
-
-static xmlNodePtr
-xml_write_format_template_members (XmlParseContext *ctxt, GnmFormatTemplate const *ft)
-{
-	xmlNode *root, *child;
-	xmlNs   *ns;
-	GSList  *member;
-
-	root = xmlNewDocNode (ctxt->doc, NULL, CC2XML ("FormatTemplate"), NULL);
-	if (root == NULL)
-		return NULL;
-
-	ns = xmlNewNs (root,
-		       CC2XML ("http://www.gnome.org/gnumeric/format-template/v1"),
-		       CC2XML ("gmr"));
-	xmlSetNs (root, ns);
-	ctxt->ns = ns;
-
-	child = xmlNewChild (root, ns, CC2XML ("Information"), NULL);
-	go_xml_node_set_cstr (child, "author", ft->author);
-	go_xml_node_set_cstr (child, "name", ft->name);
-	go_xml_node_set_cstr (child, "description", ft->description);
-
-	child = xmlNewChild (root, ns, CC2XML ("Members"), NULL);
-	for (member = ft->members ; member != NULL ; member = member->next)
-		xmlAddChild (child,
-			xml_write_format_template_member (ctxt, member->data));
-
-	return root;
-}
-#endif
-
-/**
- * format_template_attach_member:
- * @ft: GnmFormatTemplate
- * @member: the new member to attach
- *
- * Attaches @member to template @ft
- **/
-void
-format_template_attach_member (GnmFormatTemplate *ft, TemplateMember *member)
-{
-	g_return_if_fail (ft != NULL);
-	g_return_if_fail (member != NULL);
-
-	/*
-	 * NOTE : Append is slower, but that's not really an issue
-	 *        here, because a GnmFormatTemplate will most likely
-	 *        not have 'that many' members anyway
-	 */
-	ft->members = g_slist_append (ft->members, member);
-}
-
-/**
- * format_template_detach_member:
- * @ft: GnmFormatTemplate
- * @member: a TemplateMember
- *
- * Detaches @member from template @ft
- **/
-void
-format_template_detach_member (GnmFormatTemplate *ft, TemplateMember *member)
-{
-	g_return_if_fail (ft != NULL);
-	g_return_if_fail (member != NULL);
-
-	ft->members = g_slist_remove (ft->members, member);
-}
 
 /**
  * format_template_compare_name:
