@@ -17,8 +17,11 @@ G_BEGIN_DECLS
 	void name (GtkAction *a, WBCGtk *wbcg)
 
 GtkWidget* gnumeric_go_error_info_dialog_new (GOErrorInfo *error);
+GtkWidget* gnumeric_go_error_info_list_dialog_new (GSList *errs);
 void       gnumeric_go_error_info_dialog_show (GtkWindow *parent,
                                             GOErrorInfo *error);
+void       gnumeric_go_error_info_list_dialog_show (GtkWindow *parent,
+						    GSList *errs);
 void       gnumeric_restore_window_geometry (GtkWindow *dialog,
 					     const char *key);
 void       gnumeric_keyed_dialog (WBCGtk *wbcg,
@@ -41,6 +44,8 @@ void gnumeric_popup_menu (GtkMenu *menu, GdkEventButton *event);
  */
 void        gnumeric_position_tooltip (GtkWidget *tip, int px, int py,
 				       gboolean horizontal);
+GtkRcStyle *gnumeric_create_tooltip_rc_style (void);
+GtkWidget  *gnumeric_create_tooltip_widget (void);
 GtkWidget  *gnumeric_create_tooltip (GtkWidget *ref_widget);
 
 GladeXML   *gnm_glade_xml_new (GOCmdContext *cc, char const *gladefile,
@@ -53,6 +58,8 @@ typedef struct {
 	int sensitive_filter;
 
 	int index;
+
+	char *allocated_name;
 } GnumericPopupMenuElement;
 
 typedef gboolean (*GnumericPopupMenuHandler) (GnumericPopupMenuElement const *e,
