@@ -1189,7 +1189,7 @@ gnm_style_set_font_bold (GnmStyle *style, gboolean bold)
 
 	elem_changed (style, MSTYLE_FONT_BOLD);
 	elem_set (style, MSTYLE_FONT_BOLD);
-	style->font_detail.bold = bold;
+	style->font_detail.bold = !!bold;
 	gnm_style_clear_font (style);
 	gnm_style_clear_pango (style);
 }
@@ -1210,7 +1210,7 @@ gnm_style_set_font_italic (GnmStyle *style, gboolean italic)
 
 	elem_changed (style, MSTYLE_FONT_ITALIC);
 	elem_set (style, MSTYLE_FONT_ITALIC);
-	style->font_detail.italic = italic;
+	style->font_detail.italic = !!italic;
 	gnm_style_clear_font (style);
 	gnm_style_clear_pango (style);
 }
@@ -1244,13 +1244,13 @@ gnm_style_get_font_uline (GnmStyle const *style)
 }
 
 void
-gnm_style_set_font_strike (GnmStyle *style, gboolean const strikethrough)
+gnm_style_set_font_strike (GnmStyle *style, gboolean strikethrough)
 {
 	g_return_if_fail (style != NULL);
 
 	elem_changed (style, MSTYLE_FONT_STRIKETHROUGH);
 	elem_set (style, MSTYLE_FONT_STRIKETHROUGH);
-	style->font_detail.strikethrough = strikethrough;
+	style->font_detail.strikethrough = !!strikethrough;
 	gnm_style_clear_pango (style);
 }
 
@@ -1445,7 +1445,7 @@ gnm_style_set_wrap_text (GnmStyle *style, gboolean f)
 
 	elem_changed (style, MSTYLE_WRAP_TEXT);
 	elem_set (style, MSTYLE_WRAP_TEXT);
-	style->wrap_text = f;
+	style->wrap_text = !!f;
 }
 
 gboolean
@@ -1482,7 +1482,7 @@ gnm_style_set_shrink_to_fit (GnmStyle *style, gboolean f)
 
 	elem_changed (style, MSTYLE_SHRINK_TO_FIT);
 	elem_set (style, MSTYLE_SHRINK_TO_FIT);
-	style->shrink_to_fit = f;
+	style->shrink_to_fit = !!f;
 }
 
 gboolean
@@ -1501,7 +1501,7 @@ gnm_style_set_contents_locked (GnmStyle *style, gboolean f)
 
 	elem_changed (style, MSTYLE_CONTENTS_LOCKED);
 	elem_set (style, MSTYLE_CONTENTS_LOCKED);
-	style->contents_locked = f;
+	style->contents_locked = !!f;
 }
 
 gboolean
@@ -1520,7 +1520,7 @@ gnm_style_set_contents_hidden (GnmStyle *style, gboolean f)
 
 	elem_changed (style, MSTYLE_CONTENTS_HIDDEN);
 	elem_set (style, MSTYLE_CONTENTS_HIDDEN);
-	style->contents_hidden = f;
+	style->contents_hidden = !!f;
 }
 
 gboolean
@@ -1648,6 +1648,7 @@ gnm_style_get_pango_attrs (GnmStyle const *style,
 			   double zoom)
 {
 	PangoAttrList *l;
+	GnmUnderline ul;
 
 	if (style->pango_attrs) {
 		if (zoom == style->pango_attrs_zoom) {
@@ -1669,19 +1670,16 @@ gnm_style_get_pango_attrs (GnmStyle const *style,
 	}
 
 	/* Handle underlining.  */
-	switch (gnm_style_get_font_uline (style)) {
-	case UNDERLINE_SINGLE :
-		add_attr (l, pango_attr_underline_new (PANGO_UNDERLINE_SINGLE));
-		break;
-	case UNDERLINE_DOUBLE :
-		add_attr (l, pango_attr_underline_new (PANGO_UNDERLINE_DOUBLE));
-		break;
-	default :
-		break;
-	}
+	ul = gnm_style_get_font_uline (style);
+	if (ul != UNDERLINE_NONE)
+		add_attr (l, 
+			  pango_attr_underline_new (gnm_translate_underline_to_pango (ul)));
 
+	/* Handle strikethrough. */
 	if (gnm_style_get_font_strike (style))
 		add_attr (l, pango_attr_strikethrough_new (TRUE));
+
+	/* Handle script. */
 	switch (gnm_style_get_font_script (style)) {
 	default :
 	case GO_FONT_SCRIPT_STANDARD :
@@ -1720,19 +1718,11 @@ gnm_style_generate_attrs_full (GnmStyle const *style)
 	add_attr (l, pango_attr_weight_new (gnm_style_get_font_bold (style)
 		? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL));
 	add_attr (l, go_color_to_pango (fore->go_color, TRUE));
-	add_attr (l, pango_attr_strikethrough_new (gnm_style_get_font_strike (style)));
-	switch (gnm_style_get_font_uline (style)) {
-	case UNDERLINE_SINGLE :
-		add_attr (l, pango_attr_underline_new (PANGO_UNDERLINE_SINGLE));
-		break;
-	case UNDERLINE_DOUBLE :
-		add_attr (l, pango_attr_underline_new (PANGO_UNDERLINE_DOUBLE));
-		break;
-	default :
-		add_attr (l, pango_attr_underline_new (PANGO_UNDERLINE_NONE));
-		break;
-	}
-
+	add_attr (l, pango_attr_strikethrough_new 
+		  (gnm_style_get_font_strike (style)));
+	add_attr (l, pango_attr_underline_new 
+		  (gnm_translate_underline_to_pango 
+		   (gnm_style_get_font_uline (style))));
 	return l;
 }
 
@@ -1782,17 +1772,9 @@ gnm_style_set_from_pango_attribute (GnmStyle *style, PangoAttribute const *attr)
 			&((PangoAttrColor *)attr)->color));
 		break;
 	case PANGO_ATTR_UNDERLINE :
-		switch (((PangoAttrInt *)attr)->value) {
-		case PANGO_UNDERLINE_NONE :
-			gnm_style_set_font_uline (style, UNDERLINE_NONE);
-			break;
-		case PANGO_UNDERLINE_SINGLE :
-			gnm_style_set_font_uline (style, UNDERLINE_SINGLE);
-			break;
-		case PANGO_UNDERLINE_DOUBLE :
-			gnm_style_set_font_uline (style, UNDERLINE_DOUBLE);
-			break;
-		}
+		gnm_style_set_font_uline 
+			(style, gnm_translate_underline_from_pango 
+			 (((PangoAttrInt *)attr)->value));
 		break;
 	case PANGO_ATTR_STRIKETHROUGH :
 		gnm_style_set_font_strike (style,
