@@ -432,13 +432,14 @@ run_solver (Sheet *sheet, WorkbookView *wbv)
 	wb_control_set_view (wbc, wbv, NULL);
 
 	/* Pick a functional algorithm.  */
-	if (!gnm_solver_factory_functional (params->options.algorithm)) {
+	if (!gnm_solver_factory_functional (params->options.algorithm,
+					    NULL)) {
 		GSList *l;
 		for (l = gnm_solver_db_get (); l; l = l->next) {
 			GnmSolverFactory *factory = l->data;
 			if (params->options.model_type != factory->type)
 				continue;
-			if (gnm_solver_factory_functional (factory)) {
+			if (gnm_solver_factory_functional (factory, NULL)) {
 				gnm_solver_param_set_algorithm (params,
 								factory);
 				break;
@@ -606,7 +607,7 @@ convert (char const *inarg, char const *outarg, char const *mergeargs[],
 					char *tmpfile;
 					int idx = 0;
 					res = 0;
-					
+
 					for (ptr = workbook_sheets(wb); ptr && !res; ptr = ptr->next, idx++) {
 						wb_view_sheet_focus(wbv, (Sheet *)ptr->data);
 						s = g_string_new (outfile);
